
const { ECS } = require("@aws-sdk/client-ecs");

class ECSClient {
    constructor (configuration) {
        this.client = new ECS(configuration);
    }

    async findCluster(name) {
        console.log(`Searching for cluster matching ${name}`);
        const clusters = await this.client.listClusters({});
        const matchingClusters = clusters.clusterArns.filter((arn) => arn.includes(name));
        // TODO print out available cluster names
        if (matchingClusters.length === 0) throw new Error(`No clusters matching ${name} were found!`);
        // TODO setup a prompt here
        if (matchingClusters.length > 1) throw new Error(`More than one cluster matching ${name} was found!`);
    
        return matchingClusters[0];
    }
    
    async findTaskLogStream(cluster, name) {
        console.log("Finding tasks");
        // TODO filter tasks here so we can bail out early
        const tasks = await this.client.listTasks({
            cluster
        });
        console.log("Describing tasks");
        const taskDetails = await this.client.describeTasks({
            cluster,
            tasks: tasks.taskArns
        });
    
        const matchingTasks = taskDetails.tasks.filter(({taskDefinitionArn}) => taskDefinitionArn.includes(name));
        // TODO print available task names
        if (matchingTasks.length === 0) throw new Error(`No tasks matching ${name} were found!`);
        
        const { taskArn, taskDefinitionArn } = matchingTasks[0];
        const [,,taskId] = taskArn.split("/");
        // console.log(matchingTasks);
        
        console.log("Describing task definition");
        const { taskDefinition } = await this.client.describeTaskDefinition({
            taskDefinition: taskDefinitionArn
        });
    
        // console.log(taskDefinition);
    
        // TODO need a more universal fix for this
        const { name: serviceName, logConfiguration } = taskDefinition.containerDefinitions.find(({ name }) => name !== "al-agent");
    
        const { options } = logConfiguration;
        const {
            ["awslogs-group"]: logGroupName,
            ["awslogs-stream-prefix"]: logStreamPrefix
        } = options;
        
        return {
            logGroupName,
            logStreamName: `${logStreamPrefix}/${serviceName}/${taskId}`
        };
    }

}

module.exports = { ECSClient };